@@ -4,7 +4,7 @@
 //  Main class for Aven.
 //
 //  Copyright (C) 2001, Mark R. Shinwell.
-//  Copyright (C) 2002,2003,2004,2005 Olly Betts
+//  Copyright (C) 2002,2003,2004,2005,2006 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /* Note: this header needs to be safe to include from C code */
@@ -28,7 +28,9 @@
 
 #include <stdarg.h>
 
-#define APP_NAME "Aven"
+#define APP_NAME wxT("Aven")
+#define APP_IMAGE wxT("aven.png")
+#define APP_ABOUT_IMAGE wxT("aven-about.png")
 
 extern
 #ifdef __cplusplus
@@ -41,9 +43,43 @@ void aven_v_report(int severity, const char *fnm, int line, int en,
 
 #include "wx.h"
 
+#include <string>
+
+inline std::string
+string_formatv(const char * fmt, va_list args)
+{
+    static size_t len = 4096;
+    static char * buf = NULL;
+    while (true) {
+	if (!buf) buf = new char[len];
+	int r = vsnprintf(buf, len, fmt, args);
+	if (r < int(len) && r != -1) {
+	    return std::string(buf, r);
+	}
+	delete [] buf;
+	buf = NULL;
+	len += len;
+    }
+}
+
+inline std::string
+string_format(const char * fmt, ...)
+{
+    va_list args;
+    va_start(args, fmt);
+    std::string s = string_formatv(fmt, args);
+    va_end(args);
+    return s;
+}
+
+// wmsg is the unicode version of msg.
+wxString wmsg(int msg_no);
+
+const wxString & wmsg_cfgpth();
+
 class MainFrm;
 
-class Aven : public wxApp {
+class Aven : public wxGLApp {
     MainFrm * m_Frame;
     // This must be a pointer, otherwise it gets initialised too early and
     // we get a segfault on MS Windows when it tries to look up paper
@@ -55,6 +91,9 @@ public:
     Aven();
     ~Aven();
 
+#ifdef __WXMSW__
+    virtual bool Initialize(int& argc, wxChar **argv);
+#endif
     virtual bool OnInit();
 
     wxPageSetupDialogData * GetPageSetupDialogData();
