@@ -4,7 +4,7 @@
 //  Tree control used for the survey tree.
 //
 //  Copyright (C) 2001, Mark R. Shinwell.
-//  Copyright (C) 2001-2003,2005 Olly Betts
+//  Copyright (C) 2001-2003,2005,2006 Olly Betts
 //  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #ifdef HAVE_CONFIG_H
@@ -33,15 +33,16 @@ BEGIN_EVENT_TABLE(AvenTreeCtrl, wxTreeCtrl)
     EVT_MOTION(AvenTreeCtrl::OnMouseMove)
     EVT_LEAVE_WINDOW(AvenTreeCtrl::OnLeaveWindow)
     EVT_TREE_SEL_CHANGED(-1, AvenTreeCtrl::OnSelChanged)
+    EVT_TREE_ITEM_ACTIVATED(-1, AvenTreeCtrl::OnItemActivated)
     EVT_CHAR(AvenTreeCtrl::OnKeyPress)
 END_EVENT_TABLE()
 
 AvenTreeCtrl::AvenTreeCtrl(MainFrm* parent, wxWindow* window_parent) :
-    wxTreeCtrl(window_parent),
+    wxTreeCtrl(window_parent, -1),
     m_Parent(parent),
     m_Enabled(false),
     m_LastItem(),
-    m_BackgroundColour(GetBackgroundColour()),
+    m_BackgroundColour(),
     m_SelValid(false)
 {
 }
@@ -56,21 +57,29 @@ void AvenTreeCtrl::OnMouseMove(wxMouseEvent& event)
 	if (!(flags & TREE_MASK)) {
 	    pos = wxTreeItemId();
 	}
-	if (pos != m_LastItem) {
-	    if (m_LastItem.IsOk()) {
-		SetItemBackgroundColour(m_LastItem, m_BackgroundColour);
-	    }
-	    if (pos.IsOk()) {
-		SetItemBackgroundColour(pos, wxColour(180, 180, 180));
-		m_Parent->DisplayTreeInfo(GetItemData(pos));
-	    } else {
-		m_Parent->DisplayTreeInfo(NULL);
-	    }
-	    m_LastItem = pos;
+	if (pos == m_LastItem) return;
+	if (pos.IsOk()) {
+	    m_Parent->DisplayTreeInfo(GetItemData(pos));
+	} else {
+	    m_Parent->DisplayTreeInfo(NULL);
 	}
     }
 }
 
+void AvenTreeCtrl::SetHere(wxTreeItemId pos)
+{
+    if (pos == m_LastItem) return;
+
+    if (m_LastItem.IsOk()) {
+	SetItemBackgroundColour(m_LastItem, m_BackgroundColour);
+    }
+    if (pos.IsOk()) {
+	m_BackgroundColour = GetItemBackgroundColour(pos);
+	SetItemBackgroundColour(pos, wxColour(180, 180, 180));
+    }
+    m_LastItem = pos;
+}
+
 void AvenTreeCtrl::OnLeaveWindow(wxMouseEvent&)
 {
     if (m_LastItem.IsOk()) {
@@ -85,16 +94,23 @@ void AvenTreeCtrl::SetEnabled(bool enabled)
     m_Enabled = enabled;
 }
 
-void AvenTreeCtrl::OnSelChanged(wxTreeEvent&)
+void AvenTreeCtrl::OnSelChanged(wxTreeEvent& e)
 {
     if (m_Enabled) {
-	m_Parent->TreeItemSelected(GetItemData(GetSelection()));
+	m_Parent->TreeItemSelected(GetItemData(e.GetItem()), false);
     }
 
     m_SelValid = true;
 }
 
-bool AvenTreeCtrl::GetSelectionData(wxTreeItemData** data)
+void AvenTreeCtrl::OnItemActivated(wxTreeEvent& e)
+{
+    if (m_Enabled) {
+	m_Parent->TreeItemSelected(GetItemData(e.GetItem()), true);
+    }
+}
+
+bool AvenTreeCtrl::GetSelectionData(wxTreeItemData** data) const
 {
     assert(m_Enabled);
     assert(data);
@@ -150,8 +166,14 @@ void AvenTreeCtrl::OnKeyPress(wxKeyEvent &e)
 	}
 	case WXK_LEFT: case WXK_RIGHT: case WXK_UP: case WXK_DOWN:
 	case WXK_HOME: case WXK_END: case WXK_PAGEUP: case WXK_PAGEDOWN:
-	// PRIOR/NEXT seem to actually be PAGEUP/PAGEDOWN (tested on wxGtk)
+	// On wx 2.6 and earlier, PRIOR/NEXT seem to actually be
+	// PAGEUP/PAGEDOWN (on wxGtk at least).  In wx 2.7 and later
+	// they're just compatibility aliases, so either they have the
+	// same value or aren't defined - either way the code won't
+	// compile.
+#if !wxCHECK_VERSION(2,7,0)
 	case WXK_PRIOR: case WXK_NEXT:
+#endif
 	    e.Skip();
 	    break;
 	default:
