@@ -2,7 +2,7 @@
  * Export to CAD-like formats (DXF, Sketch, SVG, EPS) and also Compass PLT.
  */
 
-/* Copyright (C) 1994-2004,2005 Olly Betts
+/* Copyright (C) 1994-2004,2005,2006,2008 Olly Betts
  * Copyright (C) 2004 John Pybus (SVG Output code)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /* #define DEBUG_CAD3D */
@@ -28,6 +28,8 @@
 
 #include "wx.h"
 #include <wx/utils.h>
+#include "exportfilter.h"
+#include "hpgl.h"
 #include "mainfrm.h"
 
 #include <float.h>
@@ -59,17 +61,28 @@
 #define POINTS_PER_INCH	72.0
 #define POINTS_PER_MM (POINTS_PER_INCH / MM_PER_INCH)
 
-#define SQRT_2          1.41421356237309504880168872420969
+#define SQRT_2		1.41421356237309504880168872420969
 
 #define LEGS 1
+#define SURF 2
 #define STNS 4
 #define LABELS 8
 
+static const char *layer_names[] = {
+   NULL,
+   "Legs",	// LEGS
+   "Surface",	// SURF
+   "Legs",	// LEGS|SURF
+   "Stations",	// STNS
+   NULL,
+   NULL,
+   NULL,
+   "Labels"	// LABELS
+};
+
 /* default to DXF */
 #define FMT_DEFAULT FMT_DXF
 
-static FILE *fh;
-
 /* bounds */
 static double min_x, min_y, min_z, max_x, max_y, max_z;
 
@@ -79,11 +92,38 @@ static double grid; /* grid spacing (or 0 for no grid) */
 static double scale = 500.0;
 static double factor;
 static const char *unit = "mm";
+const double SVG_MARGIN = 5.0; // In units of "unit".
 
 static const char *survey = NULL;
 
-static void
-dxf_header(const char *)
+const int *
+ExportFilter::passes() const
+{
+    static const int default_passes[] = { LEGS|SURF|STNS|LABELS, 0 };
+    return default_passes;
+}
+
+class DXF : public ExportFilter {
+  public:
+    DXF() { }
+    bool fopen(const char *fnm_out);
+    void header(const char *);
+    void line(const img_point *, const img_point *, bool, bool);
+    void label(const img_point *, const char *, bool);
+    void cross(const img_point *, bool);
+    void footer();
+};
+
+bool
+DXF::fopen(const char *fnm_out)
+{
+    // DXF gets written as text rather than binary.
+    fh = ::fopen(fnm_out, "w");
+    return (fh != NULL);
+}
+
+void
+DXF::header(const char *)
 {
    fprintf(fh, "0\nSECTION\n"
 	       "2\nHEADER\n");
@@ -201,14 +241,8 @@ dxf_header(const char *)
    }
 }
 
-static void
-dxf_start_pass(int layer)
-{
-   layer = layer;
-}
-
-static void
-dxf_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+void
+DXF::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
 {
    fPendingMove = fPendingMove; /* unused */
    fprintf(fh, "0\nLINE\n");
@@ -221,8 +255,8 @@ dxf_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMo
    fprintf(fh, "31\n%6.2f\n", p->z);
 }
 
-static void
-dxf_label(const img_point *p, const char *s, bool fSurface)
+void
+DXF::label(const img_point *p, const char *s, bool fSurface)
 {
    /* write station label to dxf file */
    fprintf(fh, "0\nTEXT\n");
@@ -234,8 +268,8 @@ dxf_label(const img_point *p, const char *s, bool fSurface)
    fprintf(fh, "1\n%s\n", s);
 }
 
-static void
-dxf_cross(const img_point *p, bool fSurface)
+void
+DXF::cross(const img_point *p, bool fSurface)
 {
    /* write station marker to dxf file */
    fprintf(fh, "0\nPOINT\n");
@@ -245,15 +279,34 @@ dxf_cross(const img_point *p, bool fSurface)
    fprintf(fh, "30\n%6.2f\n", p->z);
 }
 
-static void
-dxf_footer(void)
+void
+DXF::footer()
 {
    fprintf(fh, "000\nENDSEC\n");
    fprintf(fh, "000\nEOF\n");
 }
 
-static void
-sketch_header(const char *)
+class Sketch : public ExportFilter {
+  public:
+    Sketch() { }
+    const int * passes() const;
+    void header(const char *);
+    void start_pass(int layer);
+    void line(const img_point *, const img_point *, bool, bool);
+    void label(const img_point *, const char *, bool);
+    void cross(const img_point *, bool);
+    void footer();
+};
+
+const int *
+Sketch::passes() const
+{
+    static const int sketch_passes[] = { LEGS|SURF, STNS, LABELS, 0 };
+    return sketch_passes;
+}
+
+void
+Sketch::header(const char *)
 {
    fprintf(fh, "##Sketch 1 2\n"); /* Sketch file version */
    fprintf(fh, "document()\n");
@@ -261,22 +314,14 @@ sketch_header(const char *)
 	   (max_x - min_x) * factor, (max_y - min_y) * factor);
 }
 
-static const char *layer_names[] = {
-   NULL,
-   "Legs",
-   "Stations",
-   NULL,
-   "Labels"
-};
-
-static void
-sketch_start_pass(int layer)
+void
+Sketch::start_pass(int layer)
 {
    fprintf(fh, "layer('%s',1,1,0,0,(0,0,0))\n", layer_names[layer]);
 }
 
-static void
-sketch_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+void
+Sketch::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
 {
    fSurface = fSurface; /* unused */
    if (fPendingMove) {
@@ -286,8 +331,8 @@ sketch_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendin
    fprintf(fh, "bs(%.3f,%.3f,%.3f)\n", p->x * factor, p->y * factor, 0.0);
 }
 
-static void
-sketch_label(const img_point *p, const char *s, bool fSurface)
+void
+Sketch::label(const img_point *p, const char *s, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "fp((0,0,0))\n");
@@ -303,8 +348,8 @@ sketch_label(const img_point *p, const char *s, bool fSurface)
    fprintf(fh, "',(%.3f,%.3f))\n", p->x * factor, p->y * factor);
 }
 
-static void
-sketch_cross(const img_point *p, bool fSurface)
+void
+Sketch::cross(const img_point *p, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "b()\n");
@@ -319,8 +364,8 @@ sketch_cross(const img_point *p, bool fSurface)
 	   p->x * factor - MARKER_SIZE, p->y * factor + MARKER_SIZE, 0.0);
 }
 
-static void
-sketch_footer(void)
+void
+Sketch::footer(void)
 {
    fprintf(fh, "guidelayer('Guide Lines',1,0,0,1,(0,0,1))\n");
    if (grid) {
@@ -392,50 +437,74 @@ find_name(const img_point *p)
    return "?";
 }
 
-static bool to_close = 0;
-static bool close_g = 0;
+class SVG : public ExportFilter {
+    bool to_close;
+    bool close_g;
 
-static void
-svg_header(const char *)
+  public:
+    SVG() : to_close(false), close_g(false) { }
+    const int * passes() const;
+    void header(const char *);
+    void start_pass(int layer);
+    void line(const img_point *, const img_point *, bool, bool);
+    void label(const img_point *, const char *, bool);
+    void cross(const img_point *, bool);
+    void footer();
+};
+
+const int *
+SVG::passes() const
+{
+    static const int svg_passes[] = { LEGS|SURF, LABELS, STNS, 0 };
+    return svg_passes;
+}
+
+void
+SVG::header(const char *)
 {
    size_t i;
    htab = (point **)osmalloc(HTAB_SIZE * ossizeof(point *));
    for (i = 0; i < HTAB_SIZE; ++i) htab[i] = NULL;
    fprintf(fh, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
-   fprintf(fh, "<svg width=\"%.3f%s\" height=\"%.3f%s\""
-               " viewBox=\"0 0 %0.3f %0.3f\">\n",
-           (max_x - min_x) * factor, unit, (max_y - min_y) * factor, unit,
-           (max_x - min_x) * factor, (max_y - min_y) * factor );
+   double width = (max_x - min_x) * factor + SVG_MARGIN * 2;
+   double height = (max_y - min_y) * factor + SVG_MARGIN * 2;
+   fprintf(fh, "<svg version=\"1.1\" baseProfile=\"full\"\n"
+	       "xmlns=\"http://www.w3.org/2000/svg\"\n"
+	       "xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n"
+	       "xmlns:ev=\"http://www.w3.org/2001/xml-events\"\n"
+	       "width=\"%.3f%s\" height=\"%.3f%s\"\n"
+	       "viewBox=\"0 0 %0.3f %0.3f\">\n",
+	   width, unit, height, unit, width, height);
    fprintf(fh, "<g transform=\"translate(%.3f %.3f)\">\n",
-           min_x * -factor, max_y * factor);
-   to_close = 0;
-   close_g = 0;
+	   SVG_MARGIN - min_x * factor, SVG_MARGIN + max_y * factor);
+   to_close = false;
+   close_g = false;
 }
 
-static void
-svg_start_pass(int layer)
+void
+SVG::start_pass(int layer)
 {
    if (to_close) {
       fprintf(fh, "\"/>\n");
-      to_close = 0;
+      to_close = false;
    }
    if (close_g) {
       fprintf(fh, "</g>\n");
    }
-   close_g = 1;
-
    fprintf(fh, "<g id=\"%s\"", layer_names[layer]);
    if (layer & LEGS)
-      fprintf(fh, " style=\"stroke:black;fill:none;stroke-width:0.4\"");
+      fprintf(fh, " stroke=\"black\" fill=\"none\" stroke-width=\"0.4px\"");
    else if (layer & STNS)
-      fprintf(fh, " style=\"stroke:black;fill:none;stroke-width:0.05\"");
+      fprintf(fh, " stroke=\"black\" fill=\"none\" stroke-width=\"0.05px\"");
    else if (layer & LABELS)
-      fprintf(fh, " style=\"font-size:%.3f\"", text_height);
+      fprintf(fh, " font-size=\"%.3fem\"", text_height);
    fprintf(fh, ">\n");
+
+   close_g = true;
 }
 
-static void
-svg_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+void
+SVG::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
 {
    fSurface = fSurface; /* unused */
    if (fPendingMove) {
@@ -445,26 +514,26 @@ svg_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMo
        fprintf(fh, "<path d=\"M%.3f %.3f", p1->x * factor, p1->y * -factor);
    }
    fprintf(fh, "L%.3f %.3f", p->x * factor, p->y * -factor);
-   to_close = 1;
+   to_close = true;
 }
 
-static void
-svg_label(const img_point *p, const char *s, bool fSurface)
+void
+SVG::label(const img_point *p, const char *s, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "<text transform=\"translate(%.3f %.3f)\">",
-           p->x * factor, p->y * -factor);
-   fprintf(fh, s);
-   fprintf(fh, "</text>\n");
+	   p->x * factor, p->y * -factor);
+   fputs(s, fh);
+   fputs("</text>\n", fh);
    set_name(p, s);
 }
 
-static void
-svg_cross(const img_point *p, bool fSurface)
+void
+SVG::cross(const img_point *p, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "<circle id=\"%s\" cx=\"%.3f\" cy=\"%.3f\" r=\"%.3f\"/>\n",
-           find_name(p), p->x * factor, p->y * -factor, MARKER_SIZE * SQRT_2);
+	   find_name(p), p->x * factor, p->y * -factor, MARKER_SIZE * SQRT_2);
    fprintf(fh, "<path d=\"M%.3f %.3fL%.3f %.3fM%.3f %.3fL%.3f %.3f\"/>\n",
 	   p->x * factor - MARKER_SIZE, p->y * -factor - MARKER_SIZE,
 	   p->x * factor + MARKER_SIZE, p->y * -factor + MARKER_SIZE,
@@ -472,43 +541,54 @@ svg_cross(const img_point *p, bool fSurface)
 	   p->x * factor - MARKER_SIZE, p->y * -factor + MARKER_SIZE );
 }
 
-static void
-svg_footer(void)
+void
+SVG::footer()
 {
    if (to_close) {
       fprintf(fh, "\"/>\n");
-      to_close = 0;
+      to_close = false;
    }
    if (close_g) {
       fprintf(fh, "</g>\n");
+      close_g = false;
    }
-   close_g = 1;
-   fprintf(fh, "</g>\n</svg>");
+   fprintf(fh, "</g>\n</svg>\n");
 }
 
-static void
-plt_header(const char *title)
+class PLT : public ExportFilter {
+  public:
+    PLT() { }
+    const int * passes() const;
+    void header(const char *);
+    void line(const img_point *, const img_point *, bool, bool);
+    void label(const img_point *, const char *, bool);
+    void footer();
+};
+
+const int *
+PLT::passes() const
+{
+    static const int plt_passes[] = { LABELS, LEGS|SURF, 0 };
+    return plt_passes;
+}
+
+void
+PLT::header(const char *title)
 {
    size_t i;
    htab = (point **)osmalloc(HTAB_SIZE * ossizeof(point *));
    for (i = 0; i < HTAB_SIZE; ++i) htab[i] = NULL;
    /* Survex is E, N, Alt - PLT file is N, E, Alt */
    fprintf(fh, "Z %.3f %.3f %.3f %.3f %.3f %.3f\r\n",
-           min_y / METRES_PER_FOOT, max_y / METRES_PER_FOOT,
-           min_x / METRES_PER_FOOT, max_x / METRES_PER_FOOT,
-           min_z / METRES_PER_FOOT, max_z / METRES_PER_FOOT);
+	   min_y / METRES_PER_FOOT, max_y / METRES_PER_FOOT,
+	   min_x / METRES_PER_FOOT, max_x / METRES_PER_FOOT,
+	   min_z / METRES_PER_FOOT, max_z / METRES_PER_FOOT);
    fprintf(fh, "N%s D 1 1 1 C%s\r\n", survey ? survey : "X",
 	   (title && title[0]) ? title : "X");
 }
 
-static void
-plt_start_pass(int layer)
-{
-   layer = layer;
-}
-
-static void
-plt_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+void
+PLT::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
 {
    fSurface = fSurface; /* unused */
    if (fPendingMove) {
@@ -525,8 +605,8 @@ plt_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMo
    fprintf(fh, "S%s P -9 -9 -9 -9\r\n", find_name(p));
 }
 
-static void
-plt_label(const img_point *p, const char *s, bool fSurface)
+void
+PLT::label(const img_point *p, const char *s, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    /* FIXME: also ctrl characters - ought to remap them, not give up */
@@ -537,27 +617,30 @@ plt_label(const img_point *p, const char *s, bool fSurface)
    set_name(p, s);
 }
 
-static void
-plt_cross(const img_point *p, bool fSurface)
-{
-   fSurface = fSurface; /* unused */
-   p = p; /* unused */
-}
-
-static void
-plt_footer(void)
+void
+PLT::footer(void)
 {
    /* Survex is E, N, Alt - PLT file is N, E, Alt */
    fprintf(fh, "X %.3f %.3f %.3f %.3f %.3f %.3f\r\n",
-           min_y / METRES_PER_FOOT, max_y / METRES_PER_FOOT,
-           min_x / METRES_PER_FOOT, max_x / METRES_PER_FOOT,
-           min_z / METRES_PER_FOOT, max_z / METRES_PER_FOOT);
+	   min_y / METRES_PER_FOOT, max_y / METRES_PER_FOOT,
+	   min_x / METRES_PER_FOOT, max_x / METRES_PER_FOOT,
+	   min_z / METRES_PER_FOOT, max_z / METRES_PER_FOOT);
    /* Yucky DOS "end of textfile" marker */
    putc('\x1a', fh);
 }
 
-static void
-eps_header(const char *title)
+class EPS : public ExportFilter {
+  public:
+    EPS() { }
+    void header(const char *);
+    void line(const img_point *, const img_point *, bool, bool);
+    void label(const img_point *, const char *, bool);
+    void cross(const img_point *, bool);
+    void footer();
+};
+
+void
+EPS::header(const char *title)
 {
    const char * fontname_labels = "helvetica"; // FIXME
    int fontsize_labels = 10; // FIXME
@@ -573,19 +656,19 @@ eps_header(const char *title)
       fputs(buf, fh);
    }
 
-   wxString name;
+   string name;
 #if defined(HAVE_GETPWUID) && !defined(__DJGPP__)
    struct passwd * ent = getpwuid(getuid());
    if (ent && ent->pw_gecos[0]) name = ent->pw_gecos;
 #endif
    if (name.empty()) {
-       name = ::wxGetUserName();
+       name = ::wxGetUserName().mb_str();
        if (name.empty()) {
-	   name = ::wxGetUserId();
+	   name = ::wxGetUserId().mb_str();
        }
    }
-   if (name) {
-      fprintf(fh, "%%%%For: %s\n", name.c_str());
+   if (!name.empty()) {
+       fprintf(fh, "%%%%For: %s\n", name.c_str());
    }
 
    fprintf(fh, "%%%%BoundingBox: %d %d %d %d\n",
@@ -606,7 +689,7 @@ eps_header(const char *title)
 
    /* this code adapted from a2ps */
    fputs("%%BeginResource: encoding ISO88591Encoding\n"
-         "/ISO88591Encoding [\n", fh);
+	 "/ISO88591Encoding [\n", fh);
    fputs("/.notdef /.notdef /.notdef /.notdef\n", fh);
    fputs("/.notdef /.notdef /.notdef /.notdef\n", fh);
    fputs("/.notdef /.notdef /.notdef /.notdef\n", fh);
@@ -790,14 +873,8 @@ eps_header(const char *title)
 #endif
 }
 
-static void
-eps_start_pass(int layer)
-{
-   layer = layer;
-}
-
-static void
-eps_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
+void
+EPS::line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMove)
 {
    fSurface = fSurface; /* unused */
    if (fPendingMove) {
@@ -806,8 +883,8 @@ eps_line(const img_point *p1, const img_point *p, bool fSurface, bool fPendingMo
    fprintf(fh, "%.2f %.2f L\n", p->x, p->y);
 }
 
-static void
-eps_label(const img_point *p, const char *s, bool fSurface)
+void
+EPS::label(const img_point *p, const char *s, bool fSurface)
 {
    fprintf(fh, "%.2f %.2f M\n", p->x, p->y);
    putc('(', fh);
@@ -826,15 +903,15 @@ eps_label(const img_point *p, const char *s, bool fSurface)
    fputs(") S\n", fh);
 }
 
-static void
-eps_cross(const img_point *p, bool fSurface)
+void
+EPS::cross(const img_point *p, bool fSurface)
 {
    fSurface = fSurface; /* unused */
    fprintf(fh, "%.2f %.2f X\n", p->x, p->y);
 }
 
-static void
-eps_footer(void)
+void
+EPS::footer(void)
 {
    fputs("stroke showpage grestore\n"
 	 "%%Trailer\n"
@@ -842,62 +919,29 @@ eps_footer(void)
 	 "%%EOF\n", fh);
 }
 
-static int dxf_passes[] = { LEGS|STNS|LABELS, 0 };
-static int sketch_passes[] = { LEGS, STNS, LABELS, 0 };
-static int plt_passes[] = { LABELS, LEGS, 0 };
-static int svg_passes[] = { LEGS, LABELS, STNS, 0 };
-static int eps_passes[] = { LEGS|STNS|LABELS, 0 };
-
 typedef enum {
-    FMT_DXF = 0, FMT_SVG, FMT_SKETCH, FMT_PLT, FMT_EPS, FMT_ENDMARKER
+    FMT_DXF = 0, FMT_SVG, FMT_SKETCH, FMT_PLT, FMT_EPS, FMT_HPGL, FMT_ENDMARKER
 } export_format;
-static const char *extensions[] = { "dxf", "svg", "sk", "plt", "eps" };
+static const char *extensions[] = { "dxf", "svg", "sk", "plt", "eps", "hpgl" };
 
 bool
 Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
        double pan, double tilt, bool labels, bool crosses, bool legs,
        bool surface)
 {
-   int fSeenMove = 0;
    int fPendingMove = 0;
    img_point p, p1;
    double s = 0, c = 0;
    export_format format = FMT_DXF;
-   int *pass;
-   bool elevation = (tilt == 90.0);
-   double elev_angle = pan;
-
-   void (*header)(const char *);
-   void (*start_pass)(int);
-   void (*line)(const img_point *, const img_point *, bool, bool);
-   void (*label)(const img_point *, const char *, bool);
-   void (*cross)(const img_point *, bool);
-   void (*footer)(void);
-   const char *mode = "w"; /* default to text output */
+   const int *pass;
+   bool elevation = (tilt == 0.0);
 
    /* Defaults */
    grid = 0;
    text_height = TEXT_HEIGHT;
    marker_size = MARKER_SIZE;
+   // FIXME: allow these to be set from aven somehow!
 #if 0
-   cmdline_init(argc, argv, short_opts, long_opts, NULL, help, 1, 2);
-   while (1) {
-      int opt = cmdline_getopt();
-      if (opt == EOF) break;
-      switch (opt) {
-       case 'e': /* Elevation */
-	 elevation = 1;
-	 elev_angle = cmdline_double_arg();
-	 break;
-       case 'c': /* Crosses */
-	 crosses = 0;
-	 break;
-       case 'n': /* Labels */
-	 labels = 0;
-	 break;
-       case 'l': /* Legs */
-	 legs = 0;
-	 break;
        case 'g': /* Grid */
 	 if (optarg) {
 	    grid = cmdline_double_arg();
@@ -910,37 +954,13 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
 	 break;
        case 't': /* Text height */
 	 text_height = cmdline_double_arg();
-#ifdef DEBUG_CAD3D
-	 printf("Text Height: `%s' input, converted to %6.2f\n", optarg, text_height);
-#endif
 	 break;
        case 'm': /* Marker size */
 	 marker_size = cmdline_double_arg();
-#ifdef DEBUG_CAD3D
-	 printf("Marker Size: `%s', converted to %6.2f\n", optarg, marker_size);
-#endif
 	 break;
-       case 'D':
-	 format = FMT_DXF;
-	 break;
-       case 'S':
-	 format = FMT_SKETCH;
-	 break;
-       case 'P':
-	 format = FMT_PLT;
-	 break;
-       case 'V':
-         format = FMT_SVG;
-         break;
        case 's':
 	 survey = optarg;
 	 break;
-#ifdef DEBUG_CAD3D
-       default:
-	 printf("Internal Error: 'getopt' returned '%c' %d\n", opt, opt);
-#endif
-      }
-   }
 #endif
 
    {
@@ -949,87 +969,65 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
       for (i = 0; i < FMT_ENDMARKER; ++i) {
 	 size_t l = strlen(extensions[i]);
 	 if (len > l + 1 && fnm_out[len - l - 1] == FNM_SEP_EXT &&
-	     strcasecmp(fnm_out.c_str() + len - l, extensions[i]) == 0) {
+	     strcasecmp(fnm_out.mb_str() + len - l, extensions[i]) == 0) {
 	    format = export_format(i);
 	    break;
 	 }
       }
    }
+
+   ExportFilter * filt;
    switch (format) {
-    case FMT_DXF:
-      header = dxf_header;
-      start_pass = dxf_start_pass;
-      line = dxf_line;
-      label = dxf_label;
-      cross = dxf_cross;
-      footer = dxf_footer;
-      pass = dxf_passes;
-      break;
-    case FMT_SKETCH:
-      header = sketch_header;
-      start_pass = sketch_start_pass;
-      line = sketch_line;
-      label = sketch_label;
-      cross = sketch_cross;
-      footer = sketch_footer;
-      pass = sketch_passes;
-      factor = POINTS_PER_MM * 1000.0 / scale;
-      mode = "wb"; /* Binary file output */
-      break;
-    case FMT_PLT:
-      header = plt_header;
-      start_pass = plt_start_pass;
-      line = plt_line;
-      label = plt_label;
-      cross = plt_cross;
-      footer = plt_footer;
-      pass = plt_passes;
-      mode = "wb"; /* Binary file output */
-      break;
-    case FMT_SVG:
-      header = svg_header;
-      start_pass = svg_start_pass;
-      line = svg_line;
-      label = svg_label;
-      cross = svg_cross;
-      footer = svg_footer;
-      pass = svg_passes;
-      factor = 1000.0 / scale;
-      mode = "wb"; /* Binary file output */
-      break;
-    case FMT_EPS:
-      header = eps_header;
-      start_pass = eps_start_pass;
-      line = eps_line;
-      label = eps_label;
-      cross = eps_cross;
-      footer = eps_footer;
-      pass = eps_passes;
-      factor = POINTS_PER_MM * 1000.0 / scale;
-      mode = "wb"; /* Binary file output */
-      break;
-    default:
-      exit(1);
+       case FMT_DXF:
+	   filt = new DXF;
+	   break;
+       case FMT_SKETCH:
+	   filt = new Sketch;
+	   factor = POINTS_PER_MM * 1000.0 / scale;
+	   break;
+       case FMT_PLT:
+	   filt = new PLT;
+	   break;
+       case FMT_SVG:
+	   filt = new SVG;
+	   factor = 1000.0 / scale;
+	   break;
+       case FMT_EPS:
+	   filt = new EPS;
+	   factor = POINTS_PER_MM * 1000.0 / scale;
+	   break;
+       case FMT_HPGL:
+	   filt = new HPGL;
+	   // factor = POINTS_PER_MM * 1000.0 / scale;
+	   break;
+       default:
+	   return false;
    }
 
-   fh = fopen(fnm_out.c_str(), mode);
-   if (!fh) return false;
+   // FIXME: This should really use fn_str() - currently we probably can't
+   // save to a Unicode path on wxmsw.
+   if (!filt->fopen(fnm_out.char_str())) {
+       delete filt;
+       return false;
+   }
 
    if (elevation) {
-      s = sin(rad(elev_angle));
-      c = cos(rad(elev_angle));
+      s = sin(rad(pan));
+      c = cos(rad(pan));
    }
 
    /* Get drawing corners */
    min_x = min_y = min_z = HUGE_VAL;
    max_x = max_y = max_z = -HUGE_VAL;
-   for (int band = 0; band < mainfrm->GetNumDepthBands(); ++band) {
-	list<PointInfo*>::const_iterator pos = mainfrm->GetPoints(band);
-	list<PointInfo*>::const_iterator end = mainfrm->GetPointsEnd(band);
+   list<traverse>::const_iterator trav = mainfrm->traverses_begin();
+   list<traverse>::const_iterator tend = mainfrm->traverses_end();
+   for ( ; trav != tend; ++trav) {
+	vector<PointInfo>::const_iterator pos = trav->begin();
+	vector<PointInfo>::const_iterator end = trav->end();
 	for ( ; pos != end; ++pos) {
-	    p.x = (*pos)->GetX();
-	    p.y = (*pos)->GetY();
-	    p.z = (*pos)->GetZ();
+	    p.x = pos->GetX();
+	    p.y = pos->GetY();
+	    p.z = pos->GetZ();
 
 	    if (elevation) {
 		double xnew = p.x * c - p.y * s;
@@ -1086,30 +1084,39 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
    }
 
    /* Header */
-   header(title.c_str());
+   filt->header(title.mb_str());
 
    p1.x = p1.y = p1.z = 0; /* avoid compiler warning */
 
-   for ( ; *pass; ++pass) {
+   for (pass = filt->passes(); *pass; ++pass) {
       bool legs_this_pass = ((*pass & LEGS) && legs);
+      bool surf_this_pass = ((*pass & SURF) && legs && surface);
       bool crosses_this_pass = ((*pass & STNS) && crosses);
       bool labels_this_pass = ((*pass & LABELS) && labels);
-      if (!(legs_this_pass || crosses_this_pass || labels_this_pass))
+      if (!(legs_this_pass || surf_this_pass || crosses_this_pass ||
+	    labels_this_pass))
 	  continue;
-      start_pass(*pass);
+      filt->start_pass(*pass);
       if (legs_this_pass) {
-	 for (int band = 0; band < mainfrm->GetNumDepthBands(); ++band) {
-	     list<PointInfo*>::const_iterator pos = mainfrm->GetPoints(band);
-	     list<PointInfo*>::const_iterator end = mainfrm->GetPointsEnd(band);
+	  trav = mainfrm->traverses_begin();
+	  tend = mainfrm->traverses_end();
+	  for ( ; trav != tend; ++trav) {
+	     assert(trav->size() > 1);
+	     vector<PointInfo>::const_iterator pos = trav->begin();
+	     vector<PointInfo>::const_iterator end = trav->end();
 	     for ( ; pos != end; ++pos) {
-		 p.x = (*pos)->GetX();
-		 p.y = (*pos)->GetY();
-		 p.z = (*pos)->GetZ();
+		 p.x = pos->GetX();
+		 p.y = pos->GetY();
+		 p.z = pos->GetZ();
 
 		 if (format == FMT_SKETCH) {
 		     p.x -= min_x;
 		     p.y -= min_y;
 		     p.z -= min_z;
+		 } else if (format == FMT_HPGL) {
+		     p.x -= (min_x + max_x) / 2;
+		     p.y -= (min_y + max_y) / 2;
+		     p.z -= (min_z + max_z) / 2;
 		 }
 
 		 if (elevation) {
@@ -1120,68 +1127,110 @@ Export(const wxString &fnm_out, const wxString &title, const MainFrm * mainfrm,
 		     p.x = xnew;
 		 }
 
-		 if (!(*pos)->IsLine()) {
+		 if (pos == trav->begin()) {
+		     // First point is move...
 #ifdef DEBUG_CAD3D
 		     printf("move to %9.2f %9.2f %9.2f\n",x,y,z);
 #endif
 		     fPendingMove = 1;
-		     fSeenMove = 1;
 		 } else {
 #ifdef DEBUG_CAD3D
 		     printf("line to %9.2f %9.2f %9.2f\n", p.x, p.y, p.z);
 #endif
-		     if (!fSeenMove) {
-			 p1 = p;
-			 fPendingMove = 1;
-			 fSeenMove = 1;
-		     }
-		     if (surface || !(*pos)->IsSurface()) {
-			 line(&p1, &p, (*pos)->IsSurface(), fPendingMove);
-			 fPendingMove = 0;
-		     } else {
-			 fPendingMove = 1;
-		     }
+		     filt->line(&p1, &p, false, fPendingMove);
+		     fPendingMove = 0;
+		 }
+		 p1 = p;
+	     }
+	 }
+      }
+      if (surf_this_pass) {
+	  trav = mainfrm->surface_traverses_begin();
+	  tend = mainfrm->surface_traverses_end();
+	  for ( ; trav != tend; ++trav) {
+	     assert(trav->size() > 1);
+	     vector<PointInfo>::const_iterator pos = trav->begin();
+	     vector<PointInfo>::const_iterator end = trav->end();
+	     for ( ; pos != end; ++pos) {
+		 p.x = pos->GetX();
+		 p.y = pos->GetY();
+		 p.z = pos->GetZ();
+
+		 if (format == FMT_SKETCH) {
+		     p.x -= min_x;
+		     p.y -= min_y;
+		     p.z -= min_z;
+		 } else if (format == FMT_HPGL) {
+		     p.x -= (min_x + max_x) / 2;
+		     p.y -= (min_y + max_y) / 2;
+		     p.z -= (min_z + max_z) / 2;
+		 }
+
+		 if (elevation) {
+		     double xnew = p.x * c - p.y * s;
+		     double znew = - p.x * s - p.y * c;
+		     p.y = p.z;
+		     p.z = znew;
+		     p.x = xnew;
+		 }
+
+		 if (pos == trav->begin()) {
+		     // First point is move...
+#ifdef DEBUG_CAD3D
+		     printf("surface move to %9.2f %9.2f %9.2f\n",x,y,z);
+#endif
+		     fPendingMove = 1;
+		 } else {
+#ifdef DEBUG_CAD3D
+		     printf("surface line to %9.2f %9.2f %9.2f\n", p.x, p.y, p.z);
+#endif
+		     filt->line(&p1, &p, true, fPendingMove);
+		     fPendingMove = 0;
 		 }
 		 p1 = p;
 	     }
 	 }
       }
       if (crosses_this_pass || labels_this_pass) {
-	list<LabelInfo*>::const_iterator pos = mainfrm->GetLabels();
-	list<LabelInfo*>::const_iterator end = mainfrm->GetLabelsEnd();
-	for ( ; pos != end; ++pos) {
-	    p.x = (*pos)->GetX();
-	    p.y = (*pos)->GetY();
-	    p.z = (*pos)->GetZ();
+	  list<LabelInfo*>::const_iterator pos = mainfrm->GetLabels();
+	  list<LabelInfo*>::const_iterator end = mainfrm->GetLabelsEnd();
+	  for ( ; pos != end; ++pos) {
+	      p.x = (*pos)->GetX();
+	      p.y = (*pos)->GetY();
+	      p.z = (*pos)->GetZ();
 
-	    if (format == FMT_SKETCH) {
-	       p.x -= min_x;
-	       p.y -= min_y;
-	       p.z -= min_z;
-	    }
+	      if (format == FMT_SKETCH) {
+		  p.x -= min_x;
+		  p.y -= min_y;
+		  p.z -= min_z;
+	      } else if (format == FMT_HPGL) {
+		  p.x -= (min_x + max_x) / 2;
+		  p.y -= (min_y + max_y) / 2;
+		  p.z -= (min_z + max_z) / 2;
+	      }
 
-	    if (elevation) {
-		double xnew = p.x * c - p.y * s;
-		double znew = - p.x * s - p.y * c;
-		p.y = p.z;
-		p.z = znew;
-		p.x = xnew;
-	    }
+	      if (elevation) {
+		  double xnew = p.x * c - p.y * s;
+		  double znew = - p.x * s - p.y * c;
+		  p.y = p.z;
+		  p.z = znew;
+		  p.x = xnew;
+	      }
 #ifdef DEBUG_CAD3D
-	    printf("label '%s' at %9.2f %9.2f %9.2f\n",(*pos)->GetText(),x,y,z);
+	      printf("label '%s' at %9.2f %9.2f %9.2f\n",(*pos)->GetText(),x,y,z);
 #endif
-	    /* Use !UNDERGROUND as the criterion - we want stations where
-	     * a surface and underground survey meet to be in the
-	     * underground layer */
-	    if (labels_this_pass)
-		label(&p, (*pos)->GetText(), !(*pos)->IsUnderground());
-	    if (crosses_this_pass)
-		cross(&p, !(*pos)->IsUnderground());
-	}
+	      /* Use !UNDERGROUND as the criterion - we want stations where
+	       * a surface and underground survey meet to be in the
+	       * underground layer */
+	      if (labels_this_pass)
+		  filt->label(&p, (*pos)->GetText().mb_str(), !(*pos)->IsUnderground());
+	      if (crosses_this_pass)
+		  filt->cross(&p, !(*pos)->IsUnderground());
+	  }
       }
    }
-   footer();
-   safe_fclose(fh);
+   filt->footer();
+   delete filt;
    osfree(htab);
    htab = NULL;
    return true;
