@@ -1,5 +1,6 @@
 /* export.h
- * Export to CAD-like formats (DXF, Sketch, SVG, EPS) and also Compass PLT.
+ * Export to CAD-like formats (DXF, Sketch, SVG, EPS, HPGL) and also Compass
+ * PLT.
  */
 
 /* Copyright (C) 2004,2005 Olly Betts
@@ -19,6 +20,15 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
+#ifndef SURVEX_INCLUDED_EXPORT_H
+#define SURVEX_INCLUDED_EXPORT_H
+
+#include "wx.h"
+
+class MainFrm;
+
 bool Export(const wxString &fnm_out, const wxString &title,
 	const MainFrm * mainfrm, double pan, double tilt,
 	bool labels, bool crosses, bool legs, bool surface);
+
+#endif
