@@ -1,5 +1,5 @@
 /* OS dependent filename manipulation routines
- * Copyright (c) Olly Betts 1998-2003,2004
+ * Copyright (c) Olly Betts 1998-2003,2004,2005
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -78,7 +78,7 @@ safe_fclose(FILE *f)
 	 (void)remove(fnm);
 	 fatalerror(/*Error writing to file `%s'*/110, fnm);
       }
-      // FIXME: this case should never happen...
+      /* FIXME: this case should never happen... */
       fatalerror(/*Error writing to file*/111);
    }
 }
@@ -314,63 +314,34 @@ fopen_portable(const char *pth, const char *fnm, const char *ext,
 {
    FILE *fh = fopenWithPthAndExt(pth, fnm, ext, mode, fnmUsed);
    if (fh == NULL) {
-#if (OS==RISCOS) || (OS==UNIX)
+#if OS_UNIX
       int f_changed = 0;
       char *fnm_trans, *p;
-#if OS==RISCOS
-      char *q;
-#endif
       fnm_trans = osstrdup(fnm);
-#if OS==RISCOS
-      q = fnm_trans;
-#endif
       for (p = fnm_trans; *p; p++) {
 	 switch (*p) {
-#if (OS==RISCOS)
-	 /* swap either slash to a dot, and a dot to a forward slash */
-	 /* but .. goes to ^ */
-	 case '.':
-	    if (p[1] == '.') {
-	       *q++ = '^';
-	       p++; /* skip second dot */
-	    } else {
-	       *q++ = '/';
-	    }
-	    f_changed = 1;
-	    break;
-	 case '/': case '\\':
-	    *q++ = '.';
-	    f_changed = 1;
-	    break;
-	 default:
-	    *q++ = *p; break;
-#else
 	 case '\\': /* swap a backslash to a forward slash */
 	    *p = '/';
 	    f_changed = 1;
 	    break;
-#endif
 	 }
       }
-#if OS==RISCOS
-      *q = '\0';
-#endif
       if (f_changed)
 	 fh = fopenWithPthAndExt(pth, fnm_trans, ext, mode, fnmUsed);
 
-#if (OS==UNIX)
       /* as a last ditch measure, try lowercasing the filename */
       if (fh == NULL) {
 	 f_changed = 0;
-	 for (p = fnm_trans; *p ; p++)
-	    if (isupper((unsigned char)*p)) {
-	       *p = tolower(*p);
+	 for (p = fnm_trans; *p ; p++) {
+	    unsigned char ch = *p;
+	    if (isupper(ch)) {
+	       *p = tolower(ch);
 	       f_changed = 1;
 	    }
+	 }
 	 if (f_changed)
 	    fh = fopenWithPthAndExt(pth, fnm_trans, ext, mode, fnmUsed);
       }
-#endif
       osfree(fnm_trans);
 #endif
    }
