@@ -1,6 +1,6 @@
 /* filename.h
  * Function prototypes for filename.c
- * Copyright (C) 1998-2001 Olly Betts
+ * Copyright (C) 1998-2003 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,21 +14,25 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef FILENAME_H /* only include once */
 #define FILENAME_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include "useful.h"
 #include "osdepend.h"
 
-char * FAR path_from_fnm(const char *fnm);
-char * FAR base_from_fnm(const char *fnm);
-char * FAR baseleaf_from_fnm(const char *fnm);
-char * FAR leaf_from_fnm(const char *fnm);
-char * FAR use_path(const char *pth, const char *lf);
-char * FAR add_ext(const char *fnm, const char *ext);
+char * Far path_from_fnm(const char *fnm);
+char * Far base_from_fnm(const char *fnm);
+char * Far baseleaf_from_fnm(const char *fnm);
+char * Far leaf_from_fnm(const char *fnm);
+char * Far use_path(const char *pth, const char *lf);
+char * Far add_ext(const char *fnm, const char *ext);
 
 FILE *fopenWithPthAndExt(const char *pth, const char *fnm, const char *ext,
 			 const char *mode, char **fnmUsed);
@@ -43,4 +47,8 @@ void safe_fclose(FILE *f);
 void filename_register_output(const char *fnm);
 void filename_delete_output(void);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* FILENAME_H */
