@@ -1,6 +1,6 @@
 /* img.h
  * Header file for routines to read and write Survex ".3d" image files
- * Copyright (C) Olly Betts 1993,1994,1997,2001,2002,2003,2004,2005
+ * Copyright (C) Olly Betts 1993,1994,1997,2001,2002,2003,2004,2005,2006
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 #ifndef IMG_H
@@ -40,6 +40,8 @@ extern "C" {
 /* # define img_CROSS  2 */
 # define img_LABEL  3
 # define img_XSECT  4
+# define img_XSECT_END 5
+# define img_ERROR_INFO 6
 
 # define img_FLAG_SURFACE   0x01
 # define img_FLAG_DUPLICATE 0x02
@@ -51,6 +53,7 @@ extern "C" {
 # define img_SFLAG_EXPORTED    0x08
 # define img_SFLAG_FIXED       0x10
 
+/* No longer used: */
 # define img_XFLAG_END      0x01
 
 /* 3D coordinates (in metres) */
@@ -67,7 +70,13 @@ typedef struct {
    char separator; /* character used to separate survey levels ('.' usually) */
    time_t date1, date2;
    double l, r, u, d;
-   char * filename_opened; /* The filename actually opened (e.g. may have ".3d" added). */
+   /* Error information - valid when IMG_ERROR is returned: */
+   int n_legs;
+   double length;
+   double E, H, V;
+   /* The filename actually opened (e.g. may have ".3d" added). */
+   char * filename_opened;
+   int is_extended_elevation;
    /* all other members are for internal use only */
    FILE *fh;          /* file handle of image file */
    char *label_buf;
@@ -90,6 +99,7 @@ typedef struct {
     *   3 => prefixes for legs; compressed prefixes
     *   4 => survey date
     *   5 => LRUD info
+    *   6 => error info
     */
    int version;
    char *survey;
@@ -149,6 +159,16 @@ int img_read_item(img *pimg, img_point *p);
 void img_write_item(img *pimg, int code, int flags, const char *s,
 		    double x, double y, double z);
 
+/* Write error information for the current traverse
+ * n_legs is the number of legs in the traverse
+ * length is the traverse length (in m)
+ * E is the ratio of the observed misclosure to the theoretical one
+ * H is the ratio of the observed horizontal misclosure to the theoretical one
+ * V is the ratio of the observed vertical misclosure to the theoretical one
+ */
+void img_write_errors(img *pimg, int n_legs, double length,
+		      double E, double H, double V);
+
 /* rewind a .3d file opened for reading so the data can be read in
  * several passes
  * pimg is a pointer to an img struct returned by img_open()
