@@ -1,6 +1,6 @@
 /* matrix.c
  * Matrix building and solving routines
- * Copyright (C) 1993-2002 Olly Betts
+ * Copyright (C) 1993-2003 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*#define SOR 1*/
@@ -45,13 +45,13 @@
 #define DEBUG_MATRIX 0
 
 #if PRINT_MATRICES
-static void print_matrix(real FAR *M, real *B, long n);
+static void print_matrix(real Far *M, real *B, long n);
 #endif
 
-static void choleski(real FAR *M, real *B, long n);
+static void choleski(real Far *M, real *B, long n);
 
 #ifdef SOR
-static void sor(real FAR *M, real *B, long n);
+static void sor(real Far *M, real *B, long n);
 #endif
 
 /* for M(row, col) col must be <= row, so Y <= X */
@@ -114,7 +114,7 @@ solve_matrix(node *list)
 static void
 build_matrix(node *list)
 {
-   real FAR *M;
+   real Far *M;
    real *B;
    int dim;
 
@@ -317,7 +317,7 @@ build_matrix(node *list)
 #ifdef NO_COVARIANCES
 	    stn_tab[m]->p[dim] = B[m];
 	    if (dim == 0) {
-	       ASSERT2(pos_fixed(stn_tab[m]),
+	       SVX_ASSERT2(pos_fixed(stn_tab[m]),
 		       "setting station coordinates didn't mark pos as fixed");
 	    }
 #else
@@ -325,7 +325,7 @@ build_matrix(node *list)
 	    for (i = 0; i < 3; i++) {
 	       stn_tab[m]->p[i] = B[m * FACTOR + i];
 	    }
-	    ASSERT2(pos_fixed(stn_tab[m]),
+	    SVX_ASSERT2(pos_fixed(stn_tab[m]),
 		    "setting station coordinates didn't mark pos as fixed");
 #endif
 	 }
@@ -379,7 +379,7 @@ add_stn_to_tab(node *stn)
 /* Note M must be symmetric positive definite */
 /* routine is entitled to scribble on M and B if it wishes */
 static void
-choleski(real FAR *M, real *B, long n)
+choleski(real Far *M, real *B, long n)
 {
    int i, j, k;
 #ifndef NO_PERCENTAGE
@@ -456,7 +456,7 @@ choleski(real FAR *M, real *B, long n)
 /* Solve MX=B for X by SOR of Gauss-Siedel */
 /* routine is entitled to scribble on M and B if it wishes */
 static void
-sor(real FAR *M, real *B, long n)
+sor(real Far *M, real *B, long n)
 {
    real t, x, delta, threshold, t2;
    int row, col;
@@ -522,7 +522,7 @@ sor(real FAR *M, real *B, long n)
 
 #if PRINT_MATRICES
 static void
-print_matrix(real FAR *M, real *B, long n)
+print_matrix(real Far *M, real *B, long n)
 {
    long row, col;
    printf("Matrix, M and vector, B:\n");
