@@ -1,6 +1,6 @@
 /* message.h
  * Function prototypes for message.c
- * Copyright (C) 1998-2003 Olly Betts
+ * Copyright (C) 1998-2003,2005 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef MESSAGE_H /* only include once */
@@ -35,23 +35,27 @@ extern "C" {
 #define CHARSET_USASCII     0
 #define CHARSET_ISO_8859_1  1
 #define CHARSET_DOSCP850    2
-#define CHARSET_RISCOS31    3
+//#define CHARSET_RISCOS31    3
 #define CHARSET_UTF8	    4
 #define CHARSET_WINCP1252   5
 #define CHARSET_ISO_8859_15 6
-#define CHARSET_DOSCP437    7
+//#define CHARSET_DOSCP437    7
 #define CHARSET_ISO_8859_2  8
 #define CHARSET_WINCP1250   9
 
 extern int msg_warnings; /* keep track of how many warnings we've given */
 extern int msg_errors;   /* and how many (non-fatal) errors */
 
+/* The language code - e.g. "en_GB" */
 extern const char *msg_lang;
+/* If the language code has a country specific qualification, then this will
+ * be just the language code.  Otherwise it's NULL.  e.g. "en" */
 extern const char *msg_lang2;
 
 void msg_init(char *const *argv);
 
 const char *msg_cfgpth(void);
+const char *msg_exepth(void);
 const char *msg_appname(void);
 
 /* Message may be overwritten by next call */
