@@ -1,7 +1,7 @@
 /* netskel.c
  * Survex network reduction - remove trailing traverses and concatenate
  * traverses between junctions
- * Copyright (C) 1991-2002,2005 Olly Betts
+ * Copyright (C) 1991-2004,2005,2006 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 /* #define BLUNDER_DETECTION */
@@ -70,6 +70,7 @@ static void remove_trailing_travs(void);
 static void remove_travs(void);
 static void replace_travs(void);
 static void replace_trailing_travs(void);
+static void write_passage_models(void);
 
 static void concatenate_trav(node *stn, int i);
 
@@ -149,6 +150,9 @@ solve_network(void /*node *stnlist*/)
    validate(); dump_network();
    replace_trailing_travs();
    validate(); dump_network();
+
+   /* Now write out any passage models. */
+   write_passage_models();
 }
 
 static void
@@ -408,29 +412,10 @@ replace_travs(void)
       fhErrStat = safe_fopen_with_ext(fnm_output_base, EXT_SVX_ERRS, "w");
 
    if (!pimg) {
-      char *fnm;
-#ifdef CHASM3DX
-      if (fUseNewFormat) {
-	 fnm = add_ext(fnm_output_base, EXT_SVX_3DX);
-      } else {
-#endif
-	 fnm = add_ext(fnm_output_base, EXT_SVX_3D);
-#ifdef CHASM3DX
-      }
-#endif
+      char *fnm = add_ext(fnm_output_base, EXT_SVX_3D);
       filename_register_output(fnm);
-
-#ifdef CHASM3DX
-      if (fUseNewFormat) {
-	 pimg = cave_open_write(fnm, survey_title);
-	 if (!pimg) fatalerror(cave_error(), fnm);
-      } else {
-#endif
-	 pimg = img_open_write(fnm, survey_title, fTrue);
-	 if (!pimg) fatalerror(img_error(), fnm);
-#ifdef CHASM3DX
-      }
-#endif
+      pimg = img_open_write(fnm, survey_title, fTrue);
+      if (!pimg) fatalerror(img_error(), fnm);
       osfree(fnm);
    }
 
@@ -456,17 +441,18 @@ replace_travs(void)
 	       stn1->name->sflags |= BIT(SFLAGS_UNDERGROUND);
 	       stn2->name->sflags |= BIT(SFLAGS_UNDERGROUND);
 	    }
-#ifdef CHASM3DX
-	    if (!fUseNewFormat) {
-#endif
-	       img_write_item(pimg, img_MOVE, 0, NULL,
-			      POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
-	       img_write_item(pimg, img_LINE, leg->l.flags,
-			      sprint_prefix(stn1->name->up),
-			      POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
-#ifdef CHASM3DX
+	    img_write_item(pimg, img_MOVE, 0, NULL,
+			   POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
+	    if (leg->meta) {
+		pimg->date1 = leg->meta->date1;
+		pimg->date2 = leg->meta->date2;
+	    } else {
+		pimg->date1 = 0;
+		pimg->date2 = 0;
 	    }
-#endif
+	    img_write_item(pimg, img_LINE, leg->l.flags,
+			   sprint_prefix(stn1->name->up),
+			   POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
 	    if (!(leg->l.reverse & FLAG_ARTICULATION)) {
 #ifdef BLUNDER_DETECTION
 	       delta err;
@@ -563,14 +549,8 @@ replace_travs(void)
       eTotTheo = hTotTheo + vTotTheo;
       cLegsTrav = 0;
       lenTrav = 0.0;
-#ifdef CHASM3DX
-      if (!fUseNewFormat) {
-#endif
-	 img_write_item(pimg, img_MOVE, 0, NULL,
-			POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
-#ifdef CHASM3DX
-      }
-#endif
+      img_write_item(pimg, img_MOVE, 0, NULL,
+		     POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
 
       fArtic = stn1->leg[i]->l.reverse & FLAG_ARTICULATION;
       osfree(stn1->leg[i]);
@@ -649,17 +629,18 @@ replace_travs(void)
 		stn3->name->sflags |= BIT(SFLAGS_UNDERGROUND);
 	     }
 
-#ifdef CHASM3DX
-	    if (!fUseNewFormat) {
-#endif
-	       SVX_ASSERT(!fEquate);
-	       SVX_ASSERT(!fZeros(&leg->v));
-	       img_write_item(pimg, img_LINE, leg->l.flags,
-			      sprint_prefix(leg_pfx),
-			      POS(stn3, 0), POS(stn3, 1), POS(stn3, 2));
-#ifdef CHASM3DX
+	    SVX_ASSERT(!fEquate);
+	    SVX_ASSERT(!fZeros(&leg->v));
+	    if (leg->meta) {
+		pimg->date1 = leg->meta->date1;
+		pimg->date2 = leg->meta->date2;
+	    } else {
+		pimg->date1 = 0;
+		pimg->date2 = 0;
 	    }
-#endif
+	    img_write_item(pimg, img_LINE, leg->l.flags,
+			   sprint_prefix(leg_pfx),
+			   POS(stn3, 0), POS(stn3, 1), POS(stn3, 2));
 	 }
 
 	 /* FIXME: equate at the start of a traverse treated specially
@@ -729,10 +710,12 @@ err_stat(int cLegsTrav, double lenTrav,
 	 double hTot, double hTotTheo,
 	 double vTot, double vTotTheo)
 {
-   double sqrt_eTot;
+   double E = sqrt(eTot / eTotTheo);
+   double H = sqrt(hTot / hTotTheo);
+   double V = sqrt(vTot / vTotTheo);
    if (!fSuppress) {
+      double sqrt_eTot = sqrt(eTot);
       fputnl(fhErrStat);
-      sqrt_eTot = sqrt(eTot);
       fprintf(fhErrStat, msg(/*Original length%7.2fm (%3d legs), moved%7.2fm (%5.2fm/leg). */145),
 	      lenTrav, cLegsTrav, sqrt_eTot, sqrt_eTot / cLegsTrav);
       if (lenTrav > 0.0)
@@ -740,11 +723,11 @@ err_stat(int cLegsTrav, double lenTrav,
       else
 	 fputs(msg(/*Error    N/A*/147), fhErrStat);
       fputnl(fhErrStat);
-      fprintf(fhErrStat, "%f\n", sqrt(eTot / eTotTheo));
-      fprintf(fhErrStat, "H: %f V: %f\n",
-	      sqrt(hTot / hTotTheo), sqrt(vTot / vTotTheo));
+      fprintf(fhErrStat, "%f\n", E);
+      fprintf(fhErrStat, "H: %f V: %f\n", H, V);
       fputnl(fhErrStat);
    }
+   img_write_errors(pimg, cLegsTrav, lenTrav, E, H, V);
 }
 
 static void
@@ -782,14 +765,8 @@ replace_trailing_travs(void)
       }
       stn1->leg[i] = ptrTrail->join1;
       SVX_ASSERT(fixed(stn1));
-#ifdef CHASM3DX
-      if (!fUseNewFormat) {
-#endif
-	 img_write_item(pimg, img_MOVE, 0, NULL,
-			POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
-#ifdef CHASM3DX
-      }
-#endif
+      img_write_item(pimg, img_MOVE, 0, NULL,
+		     POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
 
       while (1) {
 	 prefix *leg_pfx;
@@ -824,18 +801,19 @@ replace_trailing_travs(void)
 		stn2->name->sflags |= BIT(SFLAGS_UNDERGROUND);
 	     }
 	 }
-#ifdef CHASM3DX
-	 if (!fUseNewFormat) {
-#endif
-	    if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
-	       SVX_ASSERT(!fZeros(&leg->v));
-	       img_write_item(pimg, img_LINE, leg->l.flags,
-			      sprint_prefix(leg_pfx),
-			      POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
+	 if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
+	    SVX_ASSERT(!fZeros(&leg->v));
+	    if (leg->meta) {
+		pimg->date1 = leg->meta->date1;
+		pimg->date2 = leg->meta->date2;
+	    } else {
+		pimg->date1 = 0;
+		pimg->date2 = 0;
 	    }
-#ifdef CHASM3DX
+	    img_write_item(pimg, img_LINE, leg->l.flags,
+			   sprint_prefix(leg_pfx),
+			   POS(stn2, 0), POS(stn2, 1), POS(stn2, 2));
 	 }
-#endif
 
 	 /* stop if not 2 node */
 	 if (!two_node(stn2)) break;
@@ -850,54 +828,49 @@ replace_trailing_travs(void)
    }
 
    /* write out connections with no survey data */
-#ifdef CHASM3DX
-   if (!fUseNewFormat) {
-#endif
-      while (nosurveyhead) {
-	 nosurveylink *p = nosurveyhead;
-	 SVX_ASSERT(fixed(p->fr));
-	 SVX_ASSERT(fixed(p->to));
-	 if (TSTBIT(p->flags, FLAGS_SURFACE)) {
-	    p->fr->name->sflags |= BIT(SFLAGS_SURFACE);
-	    p->to->name->sflags |= BIT(SFLAGS_SURFACE);
-	 } else {
-	    p->fr->name->sflags |= BIT(SFLAGS_UNDERGROUND);
-	    p->to->name->sflags |= BIT(SFLAGS_UNDERGROUND);
-	 }
-	 img_write_item(pimg, img_MOVE, 0, NULL,
-			POS(p->fr, 0), POS(p->fr, 1), POS(p->fr, 2));
-	 img_write_item(pimg, img_LINE, p->flags,
-			sprint_prefix(p->fr->name->up),
-			POS(p->to, 0), POS(p->to, 1), POS(p->to, 2));
-	 nosurveyhead = p->next;
-	 osfree(p);
+   while (nosurveyhead) {
+      nosurveylink *p = nosurveyhead;
+      SVX_ASSERT(fixed(p->fr));
+      SVX_ASSERT(fixed(p->to));
+      if (TSTBIT(p->flags, FLAGS_SURFACE)) {
+	 p->fr->name->sflags |= BIT(SFLAGS_SURFACE);
+	 p->to->name->sflags |= BIT(SFLAGS_SURFACE);
+      } else {
+	 p->fr->name->sflags |= BIT(SFLAGS_UNDERGROUND);
+	 p->to->name->sflags |= BIT(SFLAGS_UNDERGROUND);
       }
-#ifdef CHASM3DX
+      img_write_item(pimg, img_MOVE, 0, NULL,
+		     POS(p->fr, 0), POS(p->fr, 1), POS(p->fr, 2));
+      if (p->meta) {
+	  pimg->date1 = p->meta->date1;
+	  pimg->date2 = p->meta->date2;
+      } else {
+	  pimg->date1 = 0;
+	  pimg->date2 = 0;
+      }
+      img_write_item(pimg, img_LINE, p->flags,
+		     sprint_prefix(p->fr->name->up),
+		     POS(p->to, 0), POS(p->to, 1), POS(p->to, 2));
+      nosurveyhead = p->next;
+      osfree(p);
    }
-#endif
 
    /* write stations to .3d file and free legs and stations */
    FOR_EACH_STN(stn1, stnlist) {
       int d;
       SVX_ASSERT(fixed(stn1));
       /* take care of unused fixed points */
-#ifdef CHASM3DX
-      if (!fUseNewFormat) {
-#endif
-	 if (stn1->name->stn == stn1 && stn1->name->ident) {
-	    int sf = stn1->name->sflags;
-	    if (!TSTBIT(sf, SFLAGS_SOLVED)) {
-	       /* Set flag to stop station being rewritten after *solve */
-	       stn1->name->sflags = sf | BIT(SFLAGS_SOLVED);
-	       sf &= SFLAGS_MASK;
-	       if (stn1->name->max_export) sf |= BIT(SFLAGS_EXPORTED);
-	       img_write_item(pimg, img_LABEL, sf, sprint_prefix(stn1->name),
-			      POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
-	    }
+      if (stn1->name->stn == stn1 && stn1->name->ident) {
+	 int sf = stn1->name->sflags;
+	 if (!TSTBIT(sf, SFLAGS_SOLVED)) {
+	    /* Set flag to stop station being rewritten after *solve. */
+	    stn1->name->sflags = sf | BIT(SFLAGS_SOLVED);
+	    sf &= SFLAGS_MASK;
+	    if (stn1->name->max_export) sf |= BIT(SFLAGS_EXPORTED);
+	    img_write_item(pimg, img_LABEL, sf, sprint_prefix(stn1->name),
+			   POS(stn1, 0), POS(stn1, 1), POS(stn1, 2));
 	 }
-#ifdef CHASM3DX
       }
-#endif
       /* update coords of bounding box, ignoring the base positions
        * of points fixed with error estimates and only counting stations
        * in underground surveys. */
@@ -976,3 +949,49 @@ replace_trailing_travs(void)
    }
    stnlist = NULL;
 }
+
+static void
+write_passage_models(void)
+{
+   lrudlist * psg = model;
+   while (psg) {
+      lrudlist * oldp;
+      lrud * xsect = psg->tube;
+      int xflags = 0;
+      while (xsect) {
+	 lrud *oldx;
+	 prefix *pfx;
+	 const char *name;
+	 pimg->l = xsect->l;
+	 pimg->r = xsect->r;
+	 pimg->u = xsect->u;
+	 pimg->d = xsect->d;
+	 if (xsect->meta) {
+	     pimg->date1 = xsect->meta->date1;
+	     pimg->date2 = xsect->meta->date2;
+	 } else {
+	     pimg->date1 = 0;
+	     pimg->date2 = 0;
+	 }
+
+	 pfx = xsect->stn;
+	 name = sprint_prefix(pfx);
+	 oldx = xsect;
+	 xsect = xsect->next;
+	 osfree(oldx);
+
+	 if (!pfx->pos) {
+	     error_in_file(pfx->filename, pfx->line,
+			   /*Cross section specified at non-existent station `%s'*/83,
+			   name);
+	 } else {
+	     if (xsect == NULL) xflags = img_XFLAG_END;
+	     img_write_item(pimg, img_XSECT, xflags, name, 0, 0, 0);
+	 }
+      }
+      oldp = psg;
+      psg = psg->next;
+      osfree(oldp);
+   }
+   model = NULL;
+}
