@@ -1,7 +1,7 @@
 /* printwx.h */
-/* Device dependent part of Survex wxWindows driver */
+/* Device dependent part of Survex wxWidgets driver */
 /* Copyright (C) 2004 Philip Underwood
- * Copyright (C) 2004,2005 Olly Betts
+ * Copyright (C) 2004,2005,2006 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
 #include "wx.h"
@@ -23,23 +23,6 @@
 
 #include "avenprcore.h"
 
-enum {
-	svx_PRINT = 1200,
-	svx_PREVIEW,
-	svx_SCALE,
-	svx_BEARING,
-	svx_TILT,
-	svx_LEGS,
-	svx_STATIONS,
-	svx_NAMES,
-	svx_BORDERS,
-	svx_BLANKS,
-	svx_INFOBOX,
-	svx_SCALEBAR,
-	svx_PLAN,
-	svx_ELEV
-};
-
 class MainFrm;
 class wxComboBox;
 class wxStaticText;
@@ -47,11 +30,13 @@ class wxSpinCtrl;
 class wxCheckBox;
 class wxSpinEvent;
 
+// This dialog is also use for Export as well as Print.
 class svxPrintDlg : public wxDialog {
 protected:
 	layout m_layout;
 	wxComboBox* m_scale;
 	wxStaticText* m_printSize;
+	wxStaticText* m_tilttext;
 	wxSpinCtrl* m_bearing;
 	wxSpinCtrl* m_tilt;
 	wxCheckBox* m_legs;
@@ -71,14 +56,15 @@ protected:
 	svxPrintDlg(MainFrm* parent, const wxString & filename,
 		    const wxString & title, const wxString & datestamp,
 		    double angle, double tilt_angle,
-		    bool labels, bool crosses, bool legs, bool surf);
-	~svxPrintDlg();
-	void OnPrint(wxCommandEvent& event); 
+		    bool labels, bool crosses, bool legs, bool surf,
+		    bool printing);
+	void OnPrint(wxCommandEvent& event);
+	void OnExport(wxCommandEvent& event);
 	void OnPreview(wxCommandEvent& event);
 	void OnPlan(wxCommandEvent&);
 	void OnElevation(wxCommandEvent&);
-	void OnChangeSpin(wxSpinEvent& event); 
-	void OnChange(wxCommandEvent& event); 
+	void OnChangeSpin(wxSpinEvent& event);
+	void OnChange(wxCommandEvent& event);
 	void SomethingChanged();
  private:
 	DECLARE_EVENT_TABLE()
